/*
 * Created on Apr 13, 2005
 *
 * Test ship - always provides data and can always take data - 
 * internally it has a source and a sink.
 */
package com.sunlabs.archsim.fleet.components.hornAndFunnelFabric;

import java.util.Iterator;

import com.sunlabs.archsim.base.common.Parameters;
import com.sunlabs.archsim.base.hierarchy.CompositeEntity;
import com.sunlabs.archsim.base.simulation.Delay;
import com.sunlabs.archsim.fleet.common.Data;
import com.sunlabs.archsim.fleet.common.FleetGlobal;
import com.sunlabs.archsim.fleet.common.OutputFunction;
import com.sunlabs.archsim.gasp.Action;
import com.sunlabs.archsim.gasp.Component;
import com.sunlabs.archsim.gasp.Function;
import com.sunlabs.archsim.gasp.Terminal;
import com.sunlabs.archsim.gasp.terminals.InputTerminal;
import com.sunlabs.archsim.gasp.terminals.OutputTerminal;
import com.sunlabs.archsim.globals.GlobalDefaults;



/**
 *
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 * 
 * @author ib27688
 *
 * Test ship for fleet.
 */
public class TestShip extends Component {
	

		
	InputTerminal in;
	OutputTerminal ackIn;
	
	OutputTerminal out;
	InputTerminal ackOut;
	
	/** sink hidden in the ship */
	ShipSink sink;
	/** source hidden in the ship */
	ShipSource source;
	
	
	
	
	public TestShip(String name, Parameters params) {
		super(name, null);

		// process the parameters
		int srcParam= 0;
		if (params != null) {
			for (int i= 0; i<params.size(); i++) {
				String pName= params.getName(i);
				String sValue= params.getValue(i);
				
				if (pName.equals(FleetGlobal.PARAMETER) || pName.equals("src")) {
					try {
						srcParam= Integer.parseInt(sValue);
					}
					catch (Exception e) {
						director.error("Component: " + name + ", id= " + getID() 
								+ ". Bogus value used for "
								+ "src/" + FleetGlobal.PARAMETER
								+ ": " + sValue
								+ ". Value is IGNORED, default value of "
								+ srcParam
								+ " is used.");						
					}
				}
				else {
					director.error("Component: " + name + ", id= " + getID() 
							+ ": Attribute " + pName + " with value " + sValue
							+ " is ignored");					
				}
				
			}
		} // if params != null			
		
		buildTestShip(srcParam);
			
	} // TestShip
	
	protected void buildTestShip(int srcData) {
		
		in= new InputTerminal(FleetGlobal.IN, this);
		ackIn= new OutputTerminal(FleetGlobal.ACK_IN, this);
		
		sink= new ShipSink(FleetGlobal.SINK, this);
		sink.attach(in);
		sink.attach(ackIn);
		
		out= new OutputTerminal(FleetGlobal.OUT, this);
		ackOut= new InputTerminal(FleetGlobal.ACK_OUT, this);
		
		source= new ShipSource(FleetGlobal.SOURCE, this, srcData);
		source.attach(out);
		source.attach(ackOut);
		
	}
	
	public TestShip(String name) {
		super(name, null);
		
		in= new InputTerminal(FleetGlobal.IN, this);
		ackIn= new OutputTerminal(FleetGlobal.ACK_IN, this);
		
		sink= new ShipSink(FleetGlobal.SINK, this);
		sink.attach(in);
		sink.attach(ackIn);
		
		out= new OutputTerminal(FleetGlobal.OUT, this);
		ackOut= new InputTerminal(FleetGlobal.ACK_OUT, this);
		
		int data;
		String numbers= "";
		for (int i= name.length() - 1; i>= 0 ; i--) {
			if (Character.isDigit(name.charAt(i))) {
				numbers= name.charAt(i) + numbers;
			}
			else 
				break;
		} // for
		try {
			data= Integer.parseInt(numbers);
		}
		catch (NumberFormatException e) {
			data= 9999;
		}
		
		source= new ShipSource(FleetGlobal.SOURCE, this, data);
		source.attach(out);
		source.attach(ackOut);
				
	} // TestShip
	
	public TestShip(String name, int data) {
		super(name, null);
		
		in= new InputTerminal(FleetGlobal.IN, this);
		ackIn= new OutputTerminal(FleetGlobal.ACK_IN, this);
		
		sink= new ShipSink(FleetGlobal.SINK, this);
		sink.attach(in);
		sink.attach(ackIn);
		
		out= new OutputTerminal(FleetGlobal.OUT, this);
		ackOut= new InputTerminal(FleetGlobal.ACK_OUT, this);
		
		source= new ShipSource(FleetGlobal.SOURCE, this, data);
		source.attach(out);
		source.attach(ackOut);
		
		
	} // TestShip constructor
	
	public InputTerminal getIn() {
		return in;
	}
	
	public OutputTerminal getAckIn() {
		return ackIn;
	}
	
	public OutputTerminal getOut() {
		return out;
	}
	
	public InputTerminal getAckOut() {
		return ackOut;
	}
	
	/** 
	 * Get a terminal by name.
	 * Return null if no such terminal exists.
	 */
	public Terminal getTerminal(String name) {
		if (name.equals(in.getBaseName()) 
				|| name.equals(in.getBaseAlias()))
			return in;
		else if (name.equals(ackIn.getBaseName()) 
				|| name.equals(ackIn.getBaseAlias()))
			return ackIn;
		else if (name.equals(out.getBaseName())
				|| name.equals(out.getBaseAlias()))
			return out;
		else if (name.equals(ackOut.getBaseName())
				|| name.equals(ackOut.getBaseAlias()))
			return ackOut;
		else 
			return null;
			
	} // getTerminal
	
} // TestShip;

class ShipSource extends Action {

	private Integer sDelay= globals.intValue(GlobalDefaults.TEST_SHIP_SOURCE_DELAY);
	private Delay sourceDelay= (sDelay != null)
		? new Delay(sDelay.intValue())
		: GlobalDefaults.TEST_SHIP_SOURCE_DELAY_DEF;
	
	Function out;
	
	public ShipSource(String name, CompositeEntity parent, int data) {
		super(name, parent);
		out= new GenerateData(data);
		outputFunctions.put(FleetGlobal.OUT, out);
	} // FifoAction
	
	public void setDelay(Delay d) {
		sourceDelay= (Delay)d.clone();
		out.setDelay(sourceDelay);
	}
	
	/** Generate new data items */
	private class GenerateData extends OutputFunction {

		private int data;
		
		public GenerateData(int start) {
			super("GenerateData");
			this.data= start;
			functionDelay= sourceDelay;
		}
		
		public Data value() {
			if (FleetGlobal.LOG_SHIP_SOURCE) {
				Integer lLevel= globals.intValue(GlobalDefaults.LOG_LEVEL);
				int logLevel= (lLevel != null)
					? lLevel.intValue()
					: GlobalDefaults.LOG_LEVEL_DEF;
				if (logLevel == GlobalDefaults.logDetails) {
					director.journal("ShipSource " + ShipSource.this.getName() 
						+ ", id= " + ShipSource.this.getID()
						+ ", generated " + data);
				}
			}
			return new Data(data); // was data++	
		} // value
	} // GenerateData
	
}

class ShipSink extends Action {
	
	private Integer sDelay= globals.intValue(GlobalDefaults.TEST_SHIP_SINK_DELAY);
	private Delay sinkDelay= (sDelay != null)
		? new Delay(sDelay.intValue())
		: GlobalDefaults.TEST_SHIP_SINK_DELAY_DEF;
	
	private Function out= new ActionFired();
	
	public ShipSink(String name, CompositeEntity parent) {
		super(name, parent);
		outputFunctions.put(FleetGlobal.ACK_IN, out);
	} // FifoAction
	
	public void setDelay(Delay d) {
		out.setDelay(d);
	}
	
	/** Vacuous function used to acknowldge that the action has fired */ 
	private class ActionFired extends OutputFunction {
		
		public ActionFired() {
			super("Vacous");
			functionDelay= sinkDelay;
		}
	
		public Data value() {
			Iterator ti= terminals.iterator();
			Terminal t= null;
			boolean done= false;
			while (!done && ti.hasNext()) {
				t= (Terminal)(ti.next());
				if (t.getBaseName() == FleetGlobal.IN) {
					done= true;
				}
			}
			if (t != null && FleetGlobal.LOG_SHIP_SINK ) {
				Integer lLevel= globals.intValue(GlobalDefaults.LOG_LEVEL);
				int logLevel= (lLevel != null)
					? lLevel.intValue()
					: GlobalDefaults.LOG_LEVEL_DEF;				
				if (logLevel == GlobalDefaults.logDetails) {
					director.journal("Sink " + ShipSink.this.getName() 
						+ ", id= " + ShipSink.this.getID()
						+ ", received " + t.getData());
				}		
			}
			return new Data();
		} // value
	} // AcknowledgeData

} // class ShipSink