/**
 * Copyright 2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 */
package com.sunlabs.archsim.fleet.components.trunk;

import com.sunlabs.archsim.base.common.Parameters;
import com.sunlabs.archsim.base.simulation.Delay;
import com.sunlabs.archsim.fleet.common.FleetGlobal;
import com.sunlabs.archsim.fleet.components.switchFabric.Merge;
import com.sunlabs.archsim.gasp.Component;
import com.sunlabs.archsim.gasp.Terminal;
import com.sunlabs.archsim.globals.Globals;

public class Trunk extends Component {
	
	// figure out what value to produce
	Globals globals= Globals.getInstance();
	
	protected Merge trunkMerge;
	
	protected Terminal bypassIn;
	protected Terminal ackBypassIn;
	
	protected Terminal fabricIn;
	protected Terminal ackFabricIn;
	
	protected Terminal out;
	protected Terminal ackOut;
	
	
	public Trunk(String name, Parameters params){
		super(name, null);
	
		buildTrunk();
		
		// get global parameters
		// set delays: arbitration
		Integer id= globals.intValue(TrunkDefaults.TRUNK_ARBITER_DELAY);
		Delay arbiterDelay= (id != null)
			? new Delay(id.intValue())
			: TrunkDefaults.TRUNK_ARBITER_DELAY_DEF;
		
		// data copy delay
		id= globals.intValue(TrunkDefaults.TRUNK_COPY_DELAY);
		Delay copyDelay= (id != null)
			? new Delay(id.intValue())
			: TrunkDefaults.TRUNK_COPY_DELAY_DEF;
		
		// ack delay - for local ack
		id= globals.intValue(TrunkDefaults.TRUNK_ACK_DELAY);
		Delay ackDelay= (id != null)
			? new Delay(id.intValue())
			: TrunkDefaults.TRUNK_ACK_DELAY_DEF;
			
		// process specific parameters
		if (params != null) {
			for (int i= 0; i<params.size(); i++) {
				String pName= params.getName(i);
				String sValue= params.getValue(i);
				if (pName.equals(TrunkDefaults.TRUNK_ARBITER_DELAY) ) {
					try {
						int d= Integer.parseInt(sValue);
						// override the default delay
						arbiterDelay= new Delay(d);
					}
					catch (Exception e) {
						director.error("Component: " + name + ", id= " + getID() 
								+ ". Bogus value used for " + TrunkDefaults.TRUNK_ARBITER_DELAY
								+ ": " + sValue
								+ ". Value is IGNORED and the default value of "
								+ arbiterDelay + " is used.");
					}					
				} // if 
				else if (pName.equals(TrunkDefaults.TRUNK_COPY_DELAY) ) {
					try {
						int d= Integer.parseInt(sValue);
						// override the default delay
						copyDelay= new Delay(d);
					}
					catch (Exception e) {
						director.error("Component: " + name + ", id= " + getID() 
								+ ". Bogus value used for " + TrunkDefaults.TRUNK_COPY_DELAY
								+ ": " + sValue
								+ ". Value is IGNORED and the default value of "
								+ copyDelay + " is used.");
					}					
				} // if				
				else if (pName.equals(TrunkDefaults.TRUNK_ACK_DELAY) ) {
					try {
						int d= Integer.parseInt(sValue);
						// override the default delay
						ackDelay= new Delay(d);
					}
					catch (Exception e) {
						director.error("Component: " + name + ", id= " + getID() 
								+ ". Bogus value used for " + TrunkDefaults.TRUNK_ACK_DELAY
								+ ": " + sValue
								+ ". Value is IGNORED and the default value of "
								+ copyDelay + " is used.");
					}					
				} // if				
				else {
					director.error("Component: " + name + ", id= " + getID() 
							+ ": Attribute " + pName + " with value " + sValue
							+ " is ignored");					
				}

			} // for
		} // if parames != null			

		// set parameters
		// we have an individual seeting if it was provided, 
		// otherwise we have a global setting
		trunkMerge.setArbitrationDelay(arbiterDelay);
		trunkMerge.setCopyDelay(copyDelay);
		trunkMerge.setAckDelay(ackDelay);
		
	} // Trunk constructor
	
	
	protected void buildTrunk() {
		// make the merge
		trunkMerge= new Merge("TrunkMerge", 2);		
		
		// rename the terminals
		fabricIn= trunkMerge.getIn(0);
		fabricIn.setAlias(TrunkDefaults.FABRIC_IN);
		ackFabricIn= trunkMerge.getAckIn(0);
		ackFabricIn.setAlias(TrunkDefaults.ACK_FABRIC_IN);
		bypassIn= trunkMerge.getIn(1);
		bypassIn.setAlias(TrunkDefaults.BYPASS_IN);
		ackBypassIn= trunkMerge.getAckIn(1);
		ackBypassIn.setAlias(TrunkDefaults.ACK_BYPASS_IN);
		out= trunkMerge.getOut();
		out.setAlias(FleetGlobal.OUT);
		ackOut= trunkMerge.getAckOut();
		ackOut.setAlias(FleetGlobal.ACK_OUT);		
	}
	
	public Terminal getTerminal(String name) {
		if (name.equals(TrunkDefaults.FABRIC_IN)) {
			return fabricIn;
		}
		else if (name.equals(TrunkDefaults.ACK_FABRIC_IN)) {
			return ackFabricIn;
		}
		else if (name.equals(TrunkDefaults.BYPASS_IN)) {
			return bypassIn;
		}
		else if (name.equals(TrunkDefaults.ACK_BYPASS_IN)) {
			return ackBypassIn;
		}
		else if (name.equals(FleetGlobal.OUT)) {
			return out;
		}
		else if (name.equals(FleetGlobal.ACK_OUT)) {
			return ackOut;
		}
		else {
			return null;
		}
	} // getTerminal
	
} // class Trunk
