/*
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *  
 */
/*
 * Created on October 21, 2005
 *
 */
package com.sunlabs.archsim.fleet.ships.testing;

import com.sunlabs.archsim.gasp.Action;
import com.sunlabs.archsim.gasp.Component;
import com.sunlabs.archsim.gasp.Terminal;
import com.sunlabs.archsim.gasp.terminals.InputTerminal;
import com.sunlabs.archsim.gasp.terminals.OutputTerminal;

import com.sunlabs.archsim.base.common.Parameters;
import com.sunlabs.archsim.fleet.common.Data;
import com.sunlabs.archsim.fleet.common.OutputFunction;

import com.sunlabs.archsim.globals.GlobalDefaults;

/**
 * @author mh193489
 *
 */
public class TokenSource extends Component {

	OutputTerminal out;
	InputTerminal ackOut;        
    
    Action action;
    
	public TokenSource(String name, Parameters params) {
        super(name, null);
        
		// process the parameters
		if (params != null) {
			for (int i= 0; i<params.size(); i++) {
				String pName= params.getName(i);
				String sValue= params.getValue(i);
		
				director.error("Component: " + name + ", id= " + getID() 
						+ ": Attribute " + pName + " with value " + sValue
						+ " is ignored");					
		
			} // for
		} // if params != null
		        
		construct(name, 0);
	}
    
	public TokenSource(String name) {
    	super(name, null);
        construct(name, 0);				
	} // IntegerSequenceSource

        
    public void construct(String name, int start) {
        // create terminals
        out = new OutputTerminal(TestingDefaults.OUT, this);
        ackOut = new InputTerminal(TestingDefaults.ACK_OUT, this);

        // create action
        action = new TokenAction(name + "Action", start);
        
        // attach terminals to action
        action.attach(out);
        action.attach(ackOut);
    } // construct

	
	
	public OutputTerminal getOut() {
		return out;
	}
	
	public InputTerminal getAckOut() {
		return ackOut;
	}
        
        public Terminal getTerminal(String name) {
                if (name.equals(out.getBaseName())
				|| name.equals(out.getBaseAlias()))
			return out;
		else if (name.equals(ackOut.getBaseName())
				|| name.equals(ackOut.getBaseAlias()))
			return ackOut;
		else 
			return null;
			
	} // getTerminal        

        private class TokenAction extends Action {

                OutputFunction out= null;

                public TokenAction(String name, int start) {
                        super(name);
                        out= new GenerateToken();
                        outputFunctions.put(TestingDefaults.OUT, out);
                } // IntegerSequenceAction

                /** Generate new data items */
                private class GenerateToken extends OutputFunction {

                        public GenerateToken() {
                                super("GenerateData");
                                functionDelay= TestingDefaults.ACTION_DELAY;
                        }

                        public Data value() {
                                 // log to journal
                                if (GlobalDefaults.JOURNAL_ACTION_DEF) {
                                        director.journal("GenerateData " + TokenAction.this.getName() 
                                                + ", id= " + TokenAction.this.getID()
                                                + ", generated token");
                                }
                                // wrap last in Data 
                                Data data= new Data();
                                
                                return data;	
                        } // value
                } // GenerateData
        }
}
            

